// SPDX-FileCopyrightText: Copyright 2015-2025 go-swagger maintainers
// SPDX-License-Identifier: Apache-2.0

package operations

import "errors"

// ErrOperations is the sentinel error for all errors originating from the operations builder package.
var ErrOperations = errors.New("codescan:builders:operations")
