// SPDX-FileCopyrightText: Copyright 2015-2025 go-swagger maintainers
// SPDX-License-Identifier: Apache-2.0

package resolvers

import "errors"

// ErrResolver is the sentinel error for all errors originating from the resolver package.
var ErrResolver = errors.New("codescan:resolver")
