// SPDX-FileCopyrightText: Copyright 2015-2025 go-swagger maintainers
// SPDX-License-Identifier: Apache-2.0

package routes

import "errors"

// ErrRoutes is the sentinel error for all errors originating from the routes builder package.
var ErrRoutes = errors.New("codescan:builders:routes")
