// SPDX-FileCopyrightText: Copyright 2015-2025 go-swagger maintainers
// SPDX-License-Identifier: Apache-2.0

package scanner

import "errors"

// ErrScanner is the sentinel error for all errors originating from the scanner package.
var ErrScanner = errors.New("codescan:scanner")
